@@ -1,32 +1,23 @@
 /*
- * "$Id: cups-calibrate.c,v 1.5 2004/09/17 18:38:10 rleigh Exp $"
  *
  *   Super simple color calibration program for the Common UNIX
  *   Printing System.
  *
  *   Copyright 1993-2000 by Easy Software Products.
  *
- *   This program is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU General Public License,
- *   version 2, as published by the Free Software Foundation.
+ *   This program is free software; you can redistribute it and/or modify it
+ *   under the terms of the GNU General Public License as published by the Free
+ *   Software Foundation; either version 2 of the License, or (at your option)
+ *   any later version.
  *
- *   This program is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ *   This program is distributed in the hope that it will be useful, but
+ *   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ *   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, please contact Easy Software
- *   Products at:
- *
- *       Attn: CUPS Licensing Information
- *       Easy Software Products
- *       44141 Airport View Drive, Suite 204
- *       Hollywood, Maryland 20636-3111 USA
- *
- *       Voice: (301) 373-9603
- *       EMail: cups-info@cups.org
- *         WWW: http://www.cups.org
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  * Contents:
  *
@@ -103,14 +94,12 @@ main(int  argc,
   puts("This program allows you to calibrate the color output of printers");
   puts("using the Gutenprint CUPS or ESP Print Pro drivers.");
   puts("");
-  puts("Please note that this program ONLY works with the Gutenprint CUPS or");
-  puts("ESP Print Pro drivers. If you are using the Gimp-Print stp driver of");
-  puts("GhostScript or the drivers of the Print plug-in for the GIMP, this");
-  puts("calibration will not work.");
+  puts("Please note that this program ONLY works with the Gutenprint CUPS");
+  puts("driver.");
   puts("");
-  puts("These drivers by the text \"CUPS+Gutenprint\" or \"ESP Print Pro\" in");
-  puts("the model description displayed by the CUPS web interface, KUPS,");
-  puts("the ESP Print Pro Printer Manager, or printerdrake.");
+  puts("These drivers by the text \"CUPS+Gutenprint\"");
+  puts("the model description displayed by the CUPS web interface or");
+  puts("similar tool.");
   puts("");
   puts("If you are not using the correct driver, press CTRL+C now and");
   puts("reinstall your printer queue with the appropriate driver first.");
