@@ -1,6 +1,4 @@
 /*
- * "$Id: i18n.c,v 1.6 2008/08/16 16:56:06 rlk Exp $"
- *
  *   Internationalization functions for CUPS drivers.
  *
  *   Copyright 2008 Michael Sweet (mike@easysw.com)
@@ -41,6 +39,7 @@
 #include <unistd.h>
 #include <errno.h>
 #include <iconv.h>
+#include <strings.h>
 
 
 /*
@@ -87,7 +86,7 @@ static stpi_i18n_t	*stpi_pocache = NULL;
  * 'stp_i18n_load()' - Load a message catalog for a locale.
  */
 
-stp_string_list_t *			/* O - Message catalog */
+const stp_string_list_t *		/* O - Message catalog */
 stp_i18n_load(const char *locale)	/* I - Locale name */
 {
   stp_string_list_t	*po;		/* Message catalog */
@@ -244,12 +243,12 @@ stp_i18n_load(const char *locale)	/* I - Locale name */
 	  */
 
           stpi_unquote(utf8str);
-          stp_string_list_add_string(po, id, utf8str);
+          stp_string_list_add_string_unsafe(po, id, utf8str);
 	}
 	else
 	{
           stpi_unquote(str);
-          stp_string_list_add_string(po, id, str);
+          stp_string_list_add_string_unsafe(po, id, str);
         }
       }
       else if (!id[0] && str[0] && !ic)
@@ -362,12 +361,12 @@ stp_i18n_load(const char *locale)	/* I - Locale name */
       */
 
       stpi_unquote(utf8str);
-      stp_string_list_add_string(po, id, utf8str);
+      stp_string_list_add_string_unsafe(po, id, utf8str);
     }
     else
     {
       stpi_unquote(str);
-      stp_string_list_add_string(po, id, str);
+      stp_string_list_add_string_unsafe(po, id, str);
     }
   }
 
@@ -385,6 +384,8 @@ stp_i18n_load(const char *locale)	/* I - Locale name */
     stpi_pocache  = pocache;
   }
   
+  if (ic)
+    iconv_close(ic);
   return (po);
 }
 
@@ -395,7 +396,7 @@ stp_i18n_load(const char *locale)	/* I - Locale name */
 
 const char *				/* O - Localized message */
 stp_i18n_lookup(
-    stp_string_list_t *po,		/* I - Message catalog */
+    const stp_string_list_t *po,		/* I - Message catalog */
     const char        *message)		/* I - Message */
 {
   stp_param_string_t	*param;		/* Matching message */
@@ -414,7 +415,7 @@ stp_i18n_lookup(
 
 void
 stp_i18n_printf(
-    stp_string_list_t *po,		/* I - Message catalog */
+    const stp_string_list_t *po,		/* I - Message catalog */
     const char        *message,		/* I - Printf-style message */
     ...)				/* I - Additional arguments as needed */
 {
@@ -474,8 +475,3 @@ stpi_unquote(char *s)		/* IO - Original string */
 
   *d = '\0';
 }
-
-
-/*
- * End of "$Id: i18n.c,v 1.6 2008/08/16 16:56:06 rlk Exp $".
- */
