@@ -1,5 +1,4 @@
 /*
- * "$Id: dither-main.c,v 1.59 2008/02/18 14:20:17 rlk Exp $"
  *
  *   Dither routine entrypoints
  *
@@ -41,9 +40,9 @@
 
 static const stpi_dither_algorithm_t dither_algos[] =
 {
+  { "None",           N_ ("Default"),                -1 },
   /* TRANSLATORS: "EvenTone" is the proper name, rather than a */
   /* descriptive name, of this algorithm. */
-  { "None",           N_ ("Default"),                -1 },
   { "EvenTone",       N_ ("EvenTone"),               D_EVENTONE },
   { "HybridEvenTone", N_ ("Hybrid EvenTone"),        D_HYBRID_EVENTONE },
   /* Placeholders for future implementation of EvenBetter Screening */
@@ -88,16 +87,16 @@ static const unsigned sq2[] =
 static const stp_parameter_t dither_parameters[] =
 {
   {
-    "Density", N_("Density"), N_("Output Level Adjustment"),
+    "Density", N_("Density"), "Color=Yes,Category=Output Level Adjustment",
     N_("Adjust the density (amount of ink) of the print. "
        "Reduce the density if the ink bleeds through the "
        "paper or smears; increase the density if black "
        "regions are not solid."),
     STP_PARAMETER_TYPE_DOUBLE, STP_PARAMETER_CLASS_OUTPUT,
-    STP_PARAMETER_LEVEL_ADVANCED, 0, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_ADVANCED, 0, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
-    "DitherAlgorithm", N_("Dither Algorithm"), N_("Screening Adjustment"),
+    "DitherAlgorithm", N_("Dither Algorithm"), "Color=Yes,Category=Screening Adjustment",
     N_("Choose the dither algorithm to be used.\n"
        "Adaptive Hybrid usually produces the best all-around quality.\n"
        "EvenTone is a new, experimental algorithm that often produces excellent results.\n"
@@ -105,7 +104,7 @@ static const stp_parameter_t dither_parameters[] =
        "Fast and Very Fast are considerably faster, and work well for text and line art.\n"
        "Hybrid Floyd-Steinberg generally produces inferior output."),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_OUTPUT,
-    STP_PARAMETER_LEVEL_ADVANCED, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_ADVANCED, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
 };
 
@@ -254,6 +253,8 @@ stpi_set_dither_function(stp_vars_t *v)
 	  (d->x_aspect > 2 || d->y_aspect > 2))
 	d->stpi_dither_type = D_ADAPTIVE_HYBRID;
     }
+  if (d->stpi_dither_type == -1)
+    d->stpi_dither_type = D_ADAPTIVE_HYBRID;
   switch (d->stpi_dither_type)
     {
     case D_PREDITHERED:
@@ -391,20 +392,12 @@ stp_dither_init(stp_vars_t *v, stp_image_t *image, int out_width,
     }
   else
     {
-      stp_array_t *array;
-      int transposed;
-	array = stp_find_standard_dither_array(d->y_aspect, d->x_aspect);
-      transposed = d->y_aspect < d->x_aspect ? 1 : 0;
-      if (array)
-	{
-	  stp_dither_set_matrix_from_dither_array(v, array, transposed);
-	  stp_array_destroy(array);
-	}
-      else
-	{
-	  stp_eprintf(v, "Cannot find dither matrix file!  Aborting.\n");
-	  stp_abort();
-	}
+      stp_array_t *array =
+	stp_find_standard_dither_array(d->y_aspect, d->x_aspect);
+      int transposed = d->y_aspect < d->x_aspect ? 1 : 0;
+      STPI_ASSERT(array, v);
+      stp_dither_set_matrix_from_dither_array(v, array, transposed);
+      stp_array_destroy(array);
     }
 
   d->src_width = in_width;
