@@ -1,5 +1,4 @@
 /*
- * "$Id: generic-options.c,v 1.10 2006/05/28 16:59:04 rlk Exp $"
  *
  *   Copyright 2003 Robert Krawitz (rlk@alum.mit.edu)
  *
@@ -15,7 +14,7 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111STP_CHANNEL_NONE307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -59,28 +58,40 @@ static const stpi_job_mode_t standard_job_modes[] =
 static const stp_parameter_t the_parameters[] =
 {
   {
-    "Quality", N_("Print Quality"), N_("Basic Output Adjustment"),
+    "Quality", N_("Print Quality"), "Color=Yes,Category=Basic Output Adjustment",
     N_("Print Quality"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 0, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 0, 0
   },
   {
-    "ImageType", N_("Image Type"), N_("Basic Image Adjustment"),
+    "ImageType", N_("Image Type"), "Color=Yes,Category=Basic Image Adjustment",
     N_("Type of image being printed"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_OUTPUT,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 0, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 0, 0
   },
   {
-    "JobMode", N_("Job Mode"), N_("Job Mode"),
+    "JobMode", N_("Job Mode"), "Color=No,Category=Job Mode",
     N_("Job vs. page mode"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 0, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 0, 0
   },
   {
-    "PageNumber", N_("Page Number"), N_("Job Mode"),
+    "PageNumber", N_("Page Number"), "Color=No,Category=Job Mode",
     N_("Page number"),
     STP_PARAMETER_TYPE_INT, STP_PARAMETER_CLASS_CORE,
-    STP_PARAMETER_LEVEL_BASIC, 0, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
+  },
+  {
+    "NumCopies", N_("Number of Copies"), "Color=No,Category=Job Mode",
+    N_("Number of Copies"),
+    STP_PARAMETER_TYPE_INT, STP_PARAMETER_CLASS_CORE,
+    STP_PARAMETER_LEVEL_INTERNAL, 1, 0, STP_CHANNEL_NONE, 1, 0
+  },
+  {
+    "Collate", N_("Collate the Job"), "Color=No,Category=Job Mode",
+    N_("Collate the Job"),
+    STP_PARAMETER_TYPE_BOOLEAN, STP_PARAMETER_CLASS_CORE,
+    STP_PARAMETER_LEVEL_INTERNAL, 1, 0, STP_CHANNEL_NONE, 1, 0
   },
 };
 
@@ -253,5 +264,15 @@ stpi_describe_generic_parameter(const stp_vars_t *v, const char *name,
       description->bounds.integer.lower = 0;
       description->bounds.integer.upper = INT_MAX;
     }
+  else if (strcmp(name, "NumCopies") == 0)
+    {
+      description->deflt.integer = 1;
+      description->bounds.integer.lower = 1;
+      description->bounds.integer.upper = INT_MAX;
+    }
+  else if (strcmp(name, "Collate") == 0)
+    {
+      description->deflt.boolean = 0;
+    }
 }
 
