@@ -1,5 +1,4 @@
 /*
- * "$Id: print-raw.c,v 1.39 2007/08/27 00:44:12 rlk Exp $"
  *
  *   Print plug-in RAW driver for the GIMP.
  *
@@ -26,6 +25,17 @@
  * compile on generic platforms that don't support glib, gimp, gtk, etc.
  */
 
+/*
+ * To use this driver, we recommend this:
+ *
+ *   stp_set_driver(v, "raw-data-8");  // or raw_data-16 
+ *   stp_set_string_parameter(v, "PageSize", "Custom");
+ *   stp_set_page_height(v, HEIGHT);
+ *   stp_set_page_width(v, WIDTH);
+ *
+ * For further details, see  compute_thumbnail() in gutenprintui2/panel.c
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -77,16 +87,22 @@ static const int ink_count = sizeof(inks) / sizeof(ink_t);
 static const stp_parameter_t the_parameters[] =
 {
   {
-    "InkType", N_("Ink Type"), N_("Advanced Printer Setup"),
+    "InkType", N_("Ink Type"), "Color=Yes,Category=Advanced Printer Setup",
     N_("Type of ink in the printer"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_FEATURE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
   {
-    "PrintingMode", N_("Printing Mode"), N_("Core Parameter"),
+    "PrintingMode", N_("Printing Mode"), "Color=Yes,Category=Core Parameter",
     N_("Printing Output Mode"),
     STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
-    STP_PARAMETER_LEVEL_BASIC, 1, 1, -1, 1, 0
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
+  },
+  {
+    "PageSize", N_("Page Size"), "Color=No,Category=Basic Printer Setup",
+    N_("Size of the paper being printed to"),
+    STP_PARAMETER_TYPE_STRING_LIST, STP_PARAMETER_CLASS_CORE,
+    STP_PARAMETER_LEVEL_BASIC, 1, 1, STP_CHANNEL_NONE, 1, 0
   },
 };
 
@@ -138,14 +154,25 @@ raw_parameters(const stp_vars_t *v, const char *name,
       description->deflt.str =
 	stp_string_list_param(description->bounds.str, 0)->name;
     }
+  else if (strcmp(name, "PageSize") == 0)
+    {
+      int papersizes = stp_known_papersizes();
+      description->bounds.str = stp_string_list_create();
+      for (i = 0; i < papersizes; i++)
+	{
+	  /* All users of the raw drivers should use "Custom" PageSize
+	     and manually set page height/width! */
+	  const stp_papersize_t *pt = stp_get_papersize_by_index(i);
+	  stp_string_list_add_string(description->bounds.str,
+				     pt->name, gettext(pt->text));
+	}
+      description->deflt.str =
+	stp_string_list_param(description->bounds.str, 0)->name;
+    }
   else
     description->is_active = 0;
 }
 
-/*
- * 'escp2_imageable_area()' - Return the imageable area of the page.
- */
-
 static void
 raw_imageable_area(const stp_vars_t *v,
 		   int  *left,
@@ -191,9 +218,6 @@ raw_describe_output(const stp_vars_t *v)
   return "RGB";
 }
 
-/*
- * 'escp2_print()' - Print an image to an EPSON printer.
- */
 static int
 raw_print(const stp_vars_t *v, stp_image_t *image)
 {
@@ -210,18 +234,21 @@ raw_print(const stp_vars_t *v, stp_image_t *image)
   int ink_channels = 1;
   int rotate_output = 0;
   const char *ink_type = stp_get_string_parameter(nv, "InkType");
+  stp_image_init(image);
 
   stp_prune_inactive_options(nv);
   if (!stp_verify(nv))
     {
       stp_eprintf(nv, _("Print options not verified; cannot print.\n"));
       stp_vars_destroy(nv);
+      stp_image_conclude(image);
       return 0;
     }
   if (width != stp_image_width(image) || height != stp_image_height(image))
     {
       stp_eprintf(nv, _("Image dimensions must match paper dimensions"));
       stp_vars_destroy(nv);
+      stp_image_conclude(image);
       return 0;
     }
   if (ink_type)
@@ -236,7 +263,6 @@ raw_print(const stp_vars_t *v, stp_image_t *image)
 	  }
     }
 
-  stp_set_float_parameter(nv, "Density", 1.0);
   stp_set_boolean_parameter(nv, "SimpleGamma", 1);
   stp_channel_reset(nv);
   for (i = 0; i < ink_channels; i++)
@@ -251,6 +277,7 @@ raw_print(const stp_vars_t *v, stp_image_t *image)
     {
       stp_eprintf(nv, "Internal error!  Output channels or input channels must be 1\n");
       stp_vars_destroy(nv);
+      stp_image_conclude(image);
       return 0;
     }
 
