@@ -1,5 +1,4 @@
 /*
- * "$Id: testpattern.h,v 1.17 2008/01/27 21:44:21 rlk Exp $"
  *
  *   Test pattern generator for Gimp-Print
  *
@@ -37,10 +36,10 @@ typedef struct
   } type;
   union {
     struct {
-      double mins[32];
-      double vals[32];
-      double gammas[32];
-      double levels[32];
+      double mins[STP_CHANNEL_LIMIT];
+      double vals[STP_CHANNEL_LIMIT];
+      double gammas[STP_CHANNEL_LIMIT];
+      double levels[STP_CHANNEL_LIMIT];
       double lower;
       double upper;
     } pattern;
@@ -62,6 +61,13 @@ typedef struct
  */
 #define YY_ALWAYS_INTERACTIVE 1
 
+typedef enum {
+  SIZE_RELATIVE,
+  SIZE_PT,
+  SIZE_IN,
+  SIZE_MM
+} size_mode_t;
+
 extern stp_vars_t *global_vars;
 extern double global_levels[];
 extern double global_gammas[];
@@ -72,6 +78,7 @@ extern char *global_printer;
 extern double global_density;
 extern double global_xtop;
 extern double global_xleft;
+extern size_mode_t global_size_mode;
 extern double global_hsize;
 extern double global_vsize;
 extern int global_noblackline;
@@ -83,10 +90,14 @@ extern int global_did_something;
 extern int global_invert_data;
 extern int global_noscale;
 extern char *global_output;
-
+extern int global_quiet;
+extern FILE *output;
+extern int start_job;
+extern int end_job;
 
 extern char *c_strdup(const char *s);
 extern testpattern_t *get_next_testpattern(void);
+extern void close_output(void);
 
 typedef struct yylv {
   int ival;
