@@ -1,5 +1,4 @@
 /*
- * "$Id: curve.c,v 1.22 2004/09/17 18:38:28 rleigh Exp $"
  *
  *   Copyright 2002 Robert Krawitz (rlk@alum.mit.edu)
  *
@@ -29,6 +28,8 @@
 #include <getopt.h>
 #endif
 
+#pragma GCC diagnostic ignored "-Woverlength-strings"
+
 #define DEBUG_SIGNAL
 #define MIN(x, y) ((x) <= (y) ? (x) : (y))
 #include <gutenprint/gutenprint.h>
@@ -115,6 +116,16 @@ static const double standard_sat_adjustment[] =
   0.48, 0.47, 0.47, 0.49, 0.49, 0.49, 0.52, 0.51, /* G */
 };
 
+static const double reverse_sat_adjustment[] =
+{
+  0.50, 0.51, 0.52, 0.49, 0.49, 0.49, 0.47, 0.47,
+  0.48, 0.52, 0.55, 0.6,  0.65, 0.7,  0.8,  0.9,
+  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,  1.0,
+  1.0,  1.03, 1.06, 1.09, 1.12, 1.15, 1.2,  1.25,
+  1.3,  1.15, 1.05, 0.95, 0.9,  0.87, 0.83, 0.8,
+  0.78, 0.79, 0.82, 0.86, 0.9 , 0.8,  0.7,  0.6,
+};
+
 static const stp_curve_point_t standard_piecewise_sat_adjustment[] =
 {
   { 0.00, 0.50},
@@ -168,6 +179,59 @@ static const stp_curve_point_t standard_piecewise_sat_adjustment[] =
   { 1.00, 2},
 };
 
+static const stp_curve_point_t reverse_piecewise_sat_adjustment[] =
+{
+  { 0.00, 0.50},
+  { 0.02, 0.6},
+  { 0.04, 0.7},
+  { 0.06, 0.8},
+  { 0.08, 0.9},
+  { 0.10, 0.86},
+  { 0.12, 0.82},
+  { 0.14, 0.79},
+  { 0.16, 0.78},
+  { 0.18, 0.8},
+  { 0.20, 0.83},
+  { 0.22, 0.87},
+  { 0.24, 0.9},
+  { 0.26, 0.95},
+  { 0.28, 1.05},
+  { 0.30, 1.15},
+  { 0.32, 0.05},
+  { 0.34, 3.95},
+  { 0.36, 0.05},
+  { 0.38, 1.15},
+  { 0.40, 1.12},
+  { 0.42, 1.09},
+  { 0.44, 1.06},
+  { 0.46, 1.03},
+  { 0.48, 1.0},
+  { 0.50, 1.0},
+  { 0.52, 1.0},
+  { 0.54, 1.0},
+  { 0.56, 1.0},
+  { 0.58, 1.0},
+  { 0.60, 1.0},
+  { 0.62, 1.0},
+  { 0.64, 1.0},
+  { 0.66, 0.9},
+  { 0.68, 0.8},
+  { 0.70, 0.7},
+  { 0.72, 0.65},
+  { 0.74, 0.6},
+  { 0.76, 0.55},
+  { 0.78, 0.52},
+  { 0.80, 0.48},
+  { 0.82, 0.47},
+  { 0.84, 0.47},
+  { 0.86, 0.49},
+  { 0.88, 0.49},
+  { 0.90, 0.49},
+  { 0.93, 0.52},
+  { 0.96, 0.51},
+  { 1.00, 2},
+};
+
 const char *small_piecewise_curve = 
 "<?xml version=\"1.0\"?>\n"
 "<gutenprint xmlns=\"http://gimp-print.sourceforge.net/xsd/gp.xsd-1.0\"\n"
@@ -434,7 +498,7 @@ piecewise_curve_checks(stp_curve_t *curve1, int resample_points, int expected)
 	{
 	  TEST_FAIL();
 	  if (!quiet)
-	    printf("Expected %d points, got %d\n", expected, count);
+	    printf("Expected %d points, got %lu\n", expected, count);
 	}
       TEST("Comparing data");
       for (i = 0; i < count; i++)
@@ -596,7 +660,7 @@ main(int argc, char **argv)
   TEST("creation of XML string from curve");
   curve1 = stp_curve_create(STP_CURVE_WRAP_AROUND);
   stp_curve_set_bounds(curve1, 0.0, 4.0);
-  stp_curve_set_data(curve1, 48, standard_sat_adjustment);
+  stp_curve_set_data(curve1, 48, reverse_sat_adjustment);
   tmp = stp_curve_write_string(curve1);
   stp_curve_destroy(curve1);
   curve1 = NULL;
@@ -953,7 +1017,7 @@ main(int argc, char **argv)
   stp_curve_set_bounds(curve1, 0.0, 4.0);
 
   SIMPLE_TEST_CHECK(stp_curve_set_data_points
-		    (curve1, 48, standard_piecewise_sat_adjustment));
+		    (curve1, 48, reverse_piecewise_sat_adjustment));
 
   TEST("Writing piecewise wrap-around curve to string");
   tmp = stp_curve_write_string(curve1);
