@@ -1,4 +1,3 @@
-/* $Id: unprint.c,v 1.39 2008/01/27 21:28:08 rlk Exp $ */
 /*
  * Generate PPM files from printer output
  *
@@ -36,6 +35,12 @@
 #define inline __inline__
 #endif
 
+typedef enum {
+  QT_NONE,
+  QT_QUAD,
+  QT_MIS
+} quadtone_t;
+
 /*
  * Printer state variable.
  */
@@ -65,7 +70,7 @@ typedef struct {
   int right_edge;
   int top_edge;
   int bottom_edge;
-  int quadtone;
+  quadtone_t quadtone;
 } pstate_t;
 
 /* We'd need about a gigabyte of ram to hold a ppm file of an 8.5 x 11
@@ -87,7 +92,7 @@ typedef struct {
  * actually read in the data.  This optimization may be worthwhile.
  */
 
-#define MAX_INKS 12
+#define MAX_INKS 20
 typedef struct {
    unsigned char *line[MAX_INKS];
    int startx[MAX_INKS];
@@ -115,10 +120,10 @@ line_type **page=NULL;
 
 /* Color Codes:
    color    Epson1  Epson2   Sequential
-   Black    0       0        0
-   Magenta  1       1        1
-   Cyan     2       2        2
-   Yellow   4       4        3
+   Black    0       0        0/16
+   Magenta  1       1        1/17
+   Cyan     2       2        2/18
+   Yellow   4       4        3/19
    L.Mag.   17      257      4
    L.Cyan   18      258      5
    L.Black  16      256      6
@@ -128,16 +133,16 @@ line_type **page=NULL;
    P.Black  64      N/A      0
    Gloss    9       N/A      10
    LL.Black 48      768      11
+   Orange   10       N/A     12
  */
 
 /* convert either Epson1 or Epson2 color encoding into a sequential encoding */
-static inline int
+static int
 seqcolor(int c)
 {
   switch (c)
     {
     case 0:
-    case 64:
       return 0;
     case 1:
       return 1;
@@ -166,6 +171,16 @@ seqcolor(int c)
     case 48:
     case 768:
       return 11;
+    case 10:
+      return 12;
+    case 64:
+      return 16;
+    case 65:
+      return 17;
+    case 66:
+      return 18;
+    case 68:
+      return 19;
     default:
       return 0;
     }
@@ -273,26 +288,37 @@ set_bits(unsigned char *p,int idx,int value)
 }
 
 static float ink_colors[MAX_INKS][4] =
-{{ 0,   0,  0,  1 },		/* K */
- { 1,  .1,  1,  1 },		/* M */
- { .1, .7, .7,  1 },		/* C */
- { 1,   1, .1,  1 },		/* Y */
- { 1,  .7,  1,  1 },		/* m */
- { .4,  1,  1,  1 },		/* c */
- { .7, .7, .7,  1 },		/* k */
- { .7, .7,  0,  1 },		/* dY */
- { 1,   0,  0,  1 },		/* R */
- { 0,   0,  1,  1 },		/* B */
- { 1,   1,  1,  1 },		/* Gloss */
- { .8, .8, .8,  1 },		/* llk */
+/* C(R) M(G) Y(B) K(W) */
+{{ 0,   0,   0,   1 },		/* 0  K */
+ { 1,    .1, 1,   1 },		/* 1  M */
+ {  .1, 1,   1,   1 },		/* 2  C */
+ { 1,   1,    .1, 1 },		/* 3  Y */
+ { 1,    .7, 1,   1 },		/* 4  m */
+ {  .7, 1,   1,   1 },		/* 5  c */
+ {  .7,  .7,  .7, 1 },		/* 6  k */
+ {  .7,  .7, 0,   1 },		/* 7  dY */
+ { 1,   0,   0,   1 },		/* 8  R */
+ { 0,   0,   1,   1 },		/* 8  B */
+ { 1,   1,   1,   1 },		/* 10 Gloss */
+ {  .8,  .8,  .8, 1 },		/* 11 llk */
+ {  .9,  .3, 0,   1 },		/* 12 Orange */
+ { 0,   0,   0,   1 },		/* 13 K */
+ { 0,   0,   0,   1 },		/* 14 K */
+ { 0,   0,   0,   1 },		/* 15 K */
+ { 0,   0,   0,   1 },		/* 16 K */
+ { 1,    .1, 1,   1 },		/* 17 M */
+ {  .1, 1,   1,   1 },		/* 18 C */
+ { 1,   1,    .1, 1 },		/* 19 Y */
 };
 
-static float quadtone_inks[] = { 0.0, .25, .5, .75 };
+static float quadtone_inks[] = { 0.0, .5, .25, .75, .9, .8 };
+
+static float mis_quadtone_inks[] = { 0.0, .25, .75, .5, .55, .85 };
 
 static float bpp_shift[] = { 0, 1, 3, 7, 15, 31, 63, 127, 255 };
 
 static inline void
-mix_ink(ppmpixel p, int color, unsigned int amount, float *ink, int quadtone)
+mix_ink(ppmpixel p, int color, unsigned int amount, float *ink, quadtone_t quadtone)
 {
   /* this is pretty crude */
 
@@ -302,12 +328,20 @@ mix_ink(ppmpixel p, int color, unsigned int amount, float *ink, int quadtone)
       float size;
 
       size = (float) amount / bpp_shift[pstate.bpp];
-      if (quadtone)
-	for (i = 0; i < 3; i++)
-	  p[i] *= (1 - size) + size * quadtone_inks[color];
-      else
-	for (i = 0; i < 3; i++)
-	  p[i] *= (1 - size) + size * ink[i];
+      switch (quadtone)
+	{
+	case QT_QUAD:
+	  for (i = 0; i < 3; i++)
+	    p[i] *= (1 - size) + size * quadtone_inks[color];
+	  break;
+	case QT_MIS:
+	  for (i = 0; i < 3; i++)
+	    p[i] *= (1 - size) + size * mis_quadtone_inks[color];
+	  break;
+	default:
+	  for (i = 0; i < 3; i++)
+	    p[i] *= (1 - size) + size * ink[i];
+	}
     }
 }
 
@@ -432,8 +466,14 @@ write_output(FILE *fp_w, int dontwrite, int allblack)
   right = pstate.right_edge;
   height = oversample * (last - first + 1);
 
-  fprintf(stderr, "Image from (%d,%d) to (%d,%d) (%dx%d) (%.3fx%.3f)\n",
-	  left, first, right, last, right - left + 1, last - first + 1,
+  fprintf(stderr, "Image from (%d,%d) (%.3fx%.3f) to (%d,%d) (%.3fx%.3f) size (%dx%d) (%.3fx%.3f)\n",
+	  left, first - pstate.top_margin,
+	  (left / (double) pstate.page_management_units),
+	  (first / (double) pstate.page_management_units),
+	  right, last - pstate.top_margin,
+	  (right / (double) pstate.page_management_units),
+	  (last / (double) pstate.page_management_units),
+	  right - left + 1, last - first + 1,
 	  (right - left + 1) / (double) pstate.page_management_units,
 	  (last - first + 1) / (double) pstate.page_management_units);
 
@@ -444,6 +484,8 @@ write_output(FILE *fp_w, int dontwrite, int allblack)
   out_row = stp_malloc(sizeof(ppmpixel) * width);
   fprintf(stderr, "Writing output...\n");
 
+  if (dontwrite)
+    return;
   /* write out the PPM header */
   fprintf(fp_w, "P6\n");
   fprintf(fp_w, "%d %d\n", width, height);
@@ -460,24 +502,17 @@ write_output(FILE *fp_w, int dontwrite, int allblack)
 	      float *ink = ink_colors[inknum];
 	      if (lt->line[c])
 		{
-		  if (dontwrite)
-		    donothing += lt->line[c][0] ^
-		      lt->line[c][lt->stopx[c] - lt->startx[c] + 1];
-		  else
+		  for (p = lt->startx[c]; p <= lt->stopx[c]; p++)
 		    {
-		      for (p = lt->startx[c]; p <= lt->stopx[c]; p++)
-			{
-			  amount = get_bits(lt->line[c], p - lt->startx[c]);
-			  mix_ink(out_row[p - left], c, amount, ink,
-				  pstate.quadtone);
-			}
+		      amount = get_bits(lt->line[c], p - lt->startx[c]);
+		      mix_ink(out_row[p - left], c, amount, ink,
+			      pstate.quadtone);
 		    }
 		}
 	    }
 	}
-      if (!dontwrite)
-	for (i = 0; i < oversample; i++)
-	  fwrite(out_row, sizeof(ppmpixel), width, fp_w);
+      for (i = 0; i < oversample; i++)
+	fwrite(out_row, sizeof(ppmpixel), width, fp_w);
     }
   stp_free(out_row);
 }
@@ -1040,6 +1075,7 @@ parse_escp2_extended(FILE *fp_r)
 
       break;
     case 's':		/* Set print speed */
+    case 'm':		/* Set paper type */
       break;
     case 'S': /* set paper dimensions */
       switch (bufsize)
@@ -1576,6 +1612,17 @@ main(int argc,char *argv[])
 	      else
 		fp_r = stdin;
 	      break;
+	    case 'h':
+	      fprintf(stderr, "Usage: %s [-m mask] [-n nozzle_sep] [-s extra] [-b] [-q] [-Q] [-M] [-u] [in [out]]\n", argv[0]);
+	      fprintf(stderr, "        -m mask       Color mask to unprint\n");
+	      fprintf(stderr, "        -n nozzle_sep Nozzle separation in vertical units for old printers\n");
+	      fprintf(stderr, "        -s extra      Extra feed requirement for old printers (typically 1)\n");
+	      fprintf(stderr, "        -b            Unprint everything in black\n");
+	      fprintf(stderr, "        -q            Don't produce output\n");
+	      fprintf(stderr, "        -Q            Assume quadtone inks\n");
+	      fprintf(stderr, "        -M            Assume MIS quadtone inks\n");
+	      fprintf(stderr, "        -u            Unweave\n");
+	      return 1;
 	    case 'm':
 	      if (argv[arg][2])
 		{
@@ -1652,7 +1699,10 @@ main(int argc,char *argv[])
 	      no_output = 1;
 	      break;
 	    case 'Q':
-	      pstate.quadtone = 1;
+	      pstate.quadtone = QT_QUAD;
+	      break;
+	    case 'M':
+	      pstate.quadtone = QT_MIS;
 	      break;
 	    case 'u':
 	      unweave = 1;
